package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import java.net.Socket
import java.net.SocketException
 
/**
 * Abstract Provision Delegate for Apache Tomcat Server
 * @author <a href="mailto:ramesh.ramakrishnan@upco.co.uk">Ramesh Ramakrishnan</a>
 * @version 1.0.0
 * 
 * Description: Provision delegate handles server start/stop operation along with deploy mechanism 
 * 
 * <p>The delegate handles provisioning of Apache Tomcat Server environment. 
 *    It initialises variables required for server admin operation besides providing server status information.
 * </p>
 */
/* 
 * @ant-target doStartServer, doStopServer, doDeployApplication, doUnDeployApplication
 */
abstract class AbstractProvisionDelegate extends AbstractPluginTargetDelegate {

	/*
	 *  Apache Tomcat server name. Defaults to localhost
	 * @ant-property-name tomcat.server.name
	 */	
	protected String serverName
	/*
	 * Apache Tomcat server port. Defaults to 8080
	 * @ant-property-name tomcat.server.port
	 */	
	protected String serverPort
	/*
	 * Apache Tomcat manager url. takes the value of "${tomcat.server.url}/manager"
	 * @ant-property-name tomcat.server.manager.url
	 */	
	protected String serverManagerUrl
	/*
	 * Apache Tomcat user name.This should map to the "username" attribute of user element in conf/tomcat-users.xml file
	 * @ant-property-name tomcat.server.user.name
	 */	
	protected String serverUserName
	/*
	 * Apache Tomcat password. This should map to the "password" attribute of user element in conf/tomcat-users.xml file
	 * @ant-property-name tomcat.server.user.password
	 */	
	protected String serverUserPassword
	/*
	 * Time interval in seconds forcing the build to sleep. Defaults to 5
	 * @ant-property-name tomcat.server.sleep
	 */	
	protected String serverSleep
	/*
	 * Apache Tomcat environment home. This is the CATALINA_HOME environment variable value
	 * @ant-property-name tomcat.server.home
	 */	
	protected String serverHome
	/*
	 * SPAWN value of the exec task. Defaults to true
	 * @ant-property-name tomcat.server.spawn
	 */	
	protected String serverSpawn
	/*
	 * Executable value of the exec command. Defaults to cmd
	 * @ant-property-name tomcat.server.executable
	 */	
	protected String serverExecutable
	/*
	 * Directory containing server scripts
	 * @ant-property-name tomcat.server.scripts.dir
	 */	
	protected String serverScriptsDir
	
	
	
	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 AbstractProvisionDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}
	
	/**
	* Injector for Server Name element
	*/
	void setServerName( String serverName ){
		this.serverName = serverName
	}	

	/**
	* Injector for Server Port element
	*/
	void setServerPort( String serverPort ){
		this.serverPort = serverPort
	}	
	
	/**
	* Injector for Manager Url element
	*/
	void setServerManagerUrl( String serverManagerUrl ){
		this.serverManagerUrl = serverManagerUrl
	}	

	/**
	* Injector for User Name element
	*/
	void setServerUserName( String serverUserName ){
		this.serverUserName = serverUserName
	}	

	/**
	* Injector for Password element
	*/
	void setServerUserPassword( String serverUserPassword ){
		this.serverUserPassword = serverUserPassword
	}	

	/**
	* Injector for Sleep element
	*/
	void setServerSleep( String serverSleep ){
		this.serverSleep = serverSleep
	}	
	
	/**
	* Injector for ServerHome element
	*/
	void setServerHome( String serverHome ){
		this.serverHome = serverHome
	}	

	/**
	* Injector for Spawn element
	*/
	void setServerSpawn( String serverSpawn ){
		this.serverSpawn = serverSpawn
	}	
	
	/**
	* Injector for Executable element
	*/
	void setServerExecutable( String serverExecutable ){
		this.serverExecutable = serverExecutable
	}	

	/**
	* Injector for Scritps Dir element
	*/
	void setServerScriptsDir( String serverScriptsDir ){
		this.serverScriptsDir = serverScriptsDir
	}	
	
	/*
	 * Performs a socket connect and returns the server running status.
	 * returns "true" if the server is already running and "false" for a stopped instance
	 */
	boolean getServerStatus() {
		boolean status = false
		int port = Integer.parseInt(serverPort.trim())
		Socket socket
		try {
			socket = new Socket(serverName, port)
			status = true
		} catch (Exception ex) {
			status = false
		} finally {
			if (socket != null) { 
				socket.close()
			}
		}
		return status
	}
	

}